#caesar_with_dialogue
# Python script demonstrating the implementation of a simple
# Caesar shift cipher. Only the standard alphabet letters form the
# cipherbet. Other ASCII characters are ignored.
#
# In this version, the plaintext (or ciphertext) files are chosen
# using a GUI dialogue box. Multiple files can be selected.
#
# LAST UPDATED by Andy French Jan 2018

#Import timing and GUI libraries
import time
import tkinter as tk
from tkinter import filedialog

#Display greeting message
print(" A Python Caesar Shift Cryptograpy Program")
print(" By Dr Andrew French. December 2017")

#Display's today's date and time
t = time.strftime('%X %x %Z')
print(" The current time and date is " + t + "\n")

#Request from user the mapping of alphabet "a" to where it should be
#in the cipherbet.Default is "c".
print(" State starting letter of cipherbet. This is where the letter")
print(" a will map to in a circular shifted sense")
start_letter = input(" a -> ")
if start_letter == "":
    start_letter = "c"
    print( " a -> c " )

#Define lower-case alphabet
alphabet = list("abcdefghijklmnopqrstuvwxyz")
N = len(alphabet)

#Find index of lower case alphabet that corresponds to starting letter of cipherbet
index = alphabet.index( start_letter.lower() )

#Add upper case alphabet
alphabet_upper = list("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
alphabet = alphabet + alphabet_upper
  
#Determine cipherbet. Note alphabet[a:b] means elements of alphabet with indices
#running from a to b-1.
cipherbet = alphabet[index:N] + alphabet[0:index] + alphabet_upper[index:N] + alphabet_upper[0:index]

#Print string of alphabet and cipherbet
print( "\n  Alphabet = " + ''.join(alphabet) )
print( " Cipherbet = " + ''.join(cipherbet) )

#Request encryption/decryption mode
encryption_mode = input("\n Encrypt (press letter e) or decrypt (anything else) \n > ")

#Prompt user for file. Do this by creating a GUI window, and then obscuring it
root = tk.Tk()
root.withdraw()
file_paths = filedialog.askopenfilenames(filetypes=[("Text files","*.txt")])

#Store time since current epoch (1970?) in s, and hence start an internal stopwatch.
t0 = time.time()
t1 = t0

#Step through files and decrypt or encrypt
for i in range(0,len(file_paths)):
    f = file_paths[i]
    if encryption_mode == "e":
        readfile = open(f, "r")
        writefile = open(f.replace(".txt","_encrypt.txt"), "w")
        A = alphabet[:]
        C = cipherbet[:]
    else:
        #Otherwise assume in decryption mode
        readfile = open(f, "r")
        writefile = open(f.replace(".txt","_decrypt.txt"), "w")
        A = cipherbet[:]
        C = alphabet[:]

    #Create list from file content (a string)    
    s = list(readfile.read())
    t = s[:]

    #Step through string s and find index of character in list A. Exchange this in t with
    #the corresponding character in list C
    for j in range(0,len(s)):
        try:
            index = A.index( s[j] )
            t[j] = C[index]
        except:
            t[j] = s[j]    

    #Write updated file and close
    writefile.write(''.join(t))
    writefile.close()
        
    #Display completion message to console
    print("\n File : " + file_paths[i] )
    tt = time.time()
    if encryption_mode == "e": 
        print(" Encryption complete in " + str( tt - t1) + " seconds.")
    else:
        print(" Decryption complete in " + str( tt - t1) + " seconds.")

    #Update split time
    t1 = tt

#Wait for user input to close terminal window
print("\n Total processing time is: " + str(time.time() - t0) + " seconds.")
input("\n Press return to close ")

#End of code
