#vigenere
# Python script demonstrating the implementation of a
# Vigenere polyalphabetic cipher, using a defined (case insensitive)
# code word
#
# The plaintext (or ciphertext) files are chosen
# using a GUI dialogue box. Multiple files can be selected.
#
# LAST UPDATED by Andy French Jan 2018

#Import timing and GUI libraries
import time
import tkinter as tk
from tkinter import filedialog

#Display greeting message
print(" VIGENERE \n A Python Vigenere polyalphabetic Cipher Cryptograpy Program")
print(" By Dr Andrew French. January 2018")

#Display's today's date and time
t = time.strftime('%X %x %Z')
print(" The current time and date is " + t + "\n")

#Ask user for Vigenere code word
codeword = input( "\n State Vigenere code word : " ) 
codeword = list( codeword.lower() )

#Define standard lower and upper case alphabet
alphabet = "abcdefghijklmnopqrstuvwxyz"
N = len(alphabet)
alphabet_upper = list( alphabet.upper() )
alphabet = list(alphabet) + alphabet_upper

#Determine Caesar shifted alphabets with first letter being the letters in the codeword
cipherbets = []
for i in range( 0,len(codeword) ):

    #Find index of lower case alphabet that corresponds to starting letter of cipherbet
    index = alphabet.index( codeword[i] )

    #Determine cipherbet, including upper case version as well
    cipherbet = alphabet[index:N] + alphabet[0:index] + alphabet_upper[index:N] + alphabet_upper[0:index]

    #Add to cipherbets array
    cipherbets.append(cipherbet)

#Request encryption/decryption mode
encryption_mode = input("\n Encrypt (press letter e) or decrypt (anything else) \n > ")

#Prompt user for file. Do this by creating a GUI window, and then obscuring it
root = tk.Tk()
root.withdraw()
if encryption_mode == "e":
    file_paths = filedialog.askopenfilenames(title = "Choose text files to encrypt...",filetypes=[("Text files","*.txt")])
else:
    file_paths = filedialog.askopenfilenames(title = "Choose text files to decrypt...",filetypes=[("Text files","*.txt")])

#Store time since current epoch (1970?) in s, and hence start an internal stopwatch.
t0 = time.time()
t1 = t0

#Step through files and decrypt or encrypt
for i in range(0,len(file_paths)):
    f = file_paths[i]
    if encryption_mode == "e":
        readfile = open(f, "r")
        writefile = open(f.replace(".txt","_encrypt.txt"), "w")
        A = alphabet
        C = cipherbets
    else:
        #Otherwise assume in decryption mode
        readfile = open(f, "r")
        writefile = open(f.replace(".txt","_decrypt.txt"), "w")
        A = cipherbets
        C = alphabet

    #Create list from file content (a string)    
    s = list(readfile.read())
    t = s[:]

    #Step through string s and find index of character in list A. Exchange this in t with
    #the corresponding character in list C. Cycle index k through cipherbets (or alphabets)
    k = 0
    if encryption_mode == "e":
        for j in range(0,len(s)):
            try:
                index = A.index( s[j] )
                t[j] = C[k][index]
            except:
                t[j] = s[j]
            
            k = k + 1
            if k == len(codeword):
                k = 0
    else:
        for j in range(0,len(s)):
            try:
                index = A[k].index( s[j] )
                t[j] = C[index]
            except:
                t[j] = s[j]   

            k = k + 1
            if k == len(codeword):
                k = 0

    #Write updated file and close
    writefile.write(''.join(t))
    writefile.close()
        
    #Display completion message to console
    print("\n File : " + file_paths[i] )
    tt = time.time()
    if encryption_mode == "e": 
        print(" Encryption complete in " + str( tt - t1) + " seconds.")
    else:
        print(" Decryption complete in " + str( tt - t1) + " seconds.")

    #Update split time
    t1 = tt

#Wait for user input to close terminal window
print("\n Total processing time is: " + str(time.time() - t0) + " seconds.")
input("\n Press return to close ")

#End of code
