#polycipher
# Python script demonstrating the implementation of a
# polyalphabetic cipher, using a defined cipher key.
#
# The plaintext (or ciphertext) files are chosen
# using a GUI dialogue box. Multiple files can be selected.
#
# LAST UPDATED by Andy French Jan 2018

#Import timing and GUI libraries
import time
import tkinter as tk
from tkinter import filedialog

#Display greeting message
print(" POLYCIPHER \n A Python Polyalphabetic Cipher Cryptograpy Program")
print(" By Dr Andrew French. January 2018")

#Display's today's date and time
t = time.strftime('%X %x %Z')
print(" The current time and date is " + t + "\n")

#Prompt user for cipherkey filename. Do this by creating a GUI window, and then obscuring it
root = tk.Tk()
root.withdraw()
cipherkey_filename = filedialog.askopenfilename(title="Choose cipherkey file...",filetypes=[("Text files","*.txt")])

#Load cipherkey and split into alphabet and cipherbets

#Open file and read into a string 
c = open( cipherkey_filename,'r' )
ck = c.read()

#Find number of cipherbets
K = int( ck[0:3] )

#Find length of cipherbets. Note cipher key file consists of a continuous string
# First three characters define number of cipherbets (K)
# Next N characters define the alphabet
# Next K x N characters define the K cipherbets (a random shuffle of the alphabet)
ck = list(ck)
N = int( (len(ck) - 3)/(K+1) )

#Extract alphabet and cipherbets
alphabet = ck[3:3+N]
cipherbets = []
start_index = 3 + N
end_index = start_index + N
for i in range(0,K):  
    cipherbets.append( ck[ start_index : end_index ] )
    start_index = end_index
    end_index = start_index + N

#Request encryption/decryption mode
encryption_mode = input("\n Encrypt (press letter e) or decrypt (anything else) \n > ")

#Prompt user for file. Do this by creating a GUI window, and then obscuring it
root = tk.Tk()
root.withdraw()
if encryption_mode == "e":
    file_paths = filedialog.askopenfilenames(title = "Choose text files to encrypt...",filetypes=[("Text files","*.txt")])
else:
    file_paths = filedialog.askopenfilenames(title = "Choose text files to decrypt...",filetypes=[("Text files","*.txt")])

#Store time since current epoch (1970?) in s, and hence start an internal stopwatch.
t0 = time.time()
t1 = t0

#Step through files and decrypt or encrypt
for i in range(0,len(file_paths)):
    f = file_paths[i]
    if encryption_mode == "e":
        readfile = open(f, "r")
        writefile = open(f.replace(".txt","_encrypt.txt"), "w")
    else:
        #Otherwise assume in decryption mode
        readfile = open(f, "r")
        writefile = open(f.replace(".txt","_decrypt.txt"), "w")

    #Create list from file content (a string)    
    s = list(readfile.read())
    t = s[:]

    #Step through string s and find index of character in list alphabet. Exchange this in t with
    #the corresponding character in list cipherbets. Cycle index k through cipherbets
    k = 0
    if encryption_mode == "e":
        for j in range(0,len(s)):
            try:
                index = alphabet.index( s[j] )
                t[j] = cipherbets[k][index]
            except:
                t[j] = s[j]
            
            k = k + 1
            if k == K:
                k = 0
    else:
        for j in range(0,len(s)):
            try:
                index = cipherbets[k].index( s[j] )
                t[j] = alphabet[index]
            except:
                t[j] = s[j]   

            k = k + 1
            if k == K:
                k = 0

    #Write updated file and close
    writefile.write(''.join(t))
    writefile.close()
        
    #Display completion message to console
    print("\n File : " + file_paths[i] )
    tt = time.time()
    if encryption_mode == "e": 
        print(" Encryption complete in " + str( tt - t1) + " seconds.")
    else:
        print(" Decryption complete in " + str( tt - t1) + " seconds.")

    #Update split time
    t1 = tt

#Wait for user input to close terminal window
print("\n Total processing time is: " + str(time.time() - t0) + " seconds.")
input("\n Press return to close ")

#End of code
