#frequency_analysis
# Python script which performs freqency analysis on the character content of
# text files. For each file a text file is output with four numbers per line
# ASCII code,  'ASCII character', frequency, normalized fraction
#
# The text files are chosen using a GUI dialogue box.
# Multiple files can be selected.
#
# LAST UPDATED by Andy French Feb 2018

#Import libraries
import time
import tkinter as tk
from tkinter import filedialog
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as np

#Define alphabet
alphabet = list("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ")
alphabet = alphabet + list("0123456789@!£$%^*()?/\<>,.+-=][{}~#")

#Add spaces and line returns
alphabet = alphabet + [" "] + ["\n"]

#Display greeting message
print(" A Python text file frequency analysis program")
print(" By Dr Andrew French. February 2018")

#Prompt user for file. Do this by creating a GUI window, and then obscuring it
root = tk.Tk()
root.withdraw()
file_paths = filedialog.askopenfilenames(filetypes=[("Text files","*.txt")])

#Store time since current epoch (1970?) in s, and hence start an internal stopwatch.
t0 = time.time()
t1 = t0

#Step through text files and perform frequency analysis
for i in range(0,len(file_paths)):
    f = file_paths[i]
    readfile = open(f, "r")
    writefile = open(f.replace(".txt","_freq_analysis.txt"), "w")

    #Create list from file content (a string)    
    s = list(readfile.read())

    #Initialize frequency tally array
    F = np.zeros( (1,len(alphabet)) )
    F = F[0]

    #Step through string s and find index of character in list alphabet.
    #Use this to update the frequency tally array F
    for j in range(0,len(s)):
        try:
            index = alphabet.index( s[j] )
            F[index] = F[index] + 1
        except:
            index = -1

    #Determine fractional frequency
    Fr = F/sum(F)

    #Step through alphabet and construct lines of output file
    t = ''
    for j in range(0,len(alphabet)):
        new_line = str( ord( alphabet[j] ) ) + ',' + '\'' + alphabet[j]
        new_line = new_line + '\'' + ',' + str( int( F[j] ) ) + ',' + str( Fr[j] ) + '\n'
        t = t + new_line
   
    #Write text analysis file and close
    writefile.write(t)
    writefile.close()
        
    #Display completion message to console
    print("\n File : " + file_paths[i] )
    tt = time.time()
    print(" Frequency analysis complete in " + str( tt - t1) + " seconds.")

    #Update split time
    t1 = tt

#Wait for user input to close terminal window
print("\n Total processing time is: " + str(time.time() - t0) + " seconds.")
input("\n Press return to close ")

#End of code
