#rsa_gen_keys
# Generates prime numbers which constitute public and private keys for RSA encryption.
# Public keys are the ones Alice sends to Bob, which he uses to encrypt a message that
# only Alice (with her private keys) can decrypt.
#
# LAST UPDATED by Andy French Feb 2018

#Import primesieve and GUI libraries
import primesieve as pm
import numpy as np
import time

#Function which returns x given a, b such that x*a % b = 1
#Extended Euclidean Algorithm used for modular inverse:
#https://stackoverflow.com/questions/16044553/solving-a-modular-equation-python
def modinv(a, b):
    #Store original value of b as bb to avoid modification when b changes
    blist = [0]; blist[0] = b; bb = blist[0]

    #Extended Euclidean Algorithm
    x,y, u,v = 0,1, 1,0
    while a != 0:
        # Note x//y means floor "floor division"
        q,r = b//a, b%a ; m,n = x-u*q, y-v*q 
        b,a, x,y, u,v = a,r, u,v, m,n
    if b != 1:
        return None
    else:
        return x % bb
#

#Range of primes to search from
prime_nmin = 50000;
prime_nmax = 70000;

#

#Display greeting message
print(" RSA key generator")
print(" By Dr Andrew French. February 2018")

#Randomy choose Alice's primes p,q,e
i = np.random.randint( prime_nmin,prime_nmax,size=(1,3) )
i = i[0]
p = pm.nth_prime( i[0] )
q = pm.nth_prime( i[1] )
e = pm.nth_prime( i[2] )

#Compute prime product N that will be sent to Bob, alongside e
N = p*q

#Solve equation for d
d = modinv( e, (p-1)*(q-1) )

#Print Alice's numbers
print( '\n Alice has numbers: \n p = ' + str(p) + '\n q = ' + str(q) +
'\n N = ' + str(N) + '\n e = ' + str(e) + '\n d = ' + str(d) )

#Get current date and time and hence construct filename for keys
t = time.strftime('%X %x %Z')
t = t.replace(":","")
t = t.replace("/","")
t = t.replace(" ","")
t = t[0:11]

#Construct public and private key filenames
public_keyname = "public_key_N_e " + t + ".txt"
private_keyname = "private_key_p_q_e_d " + t + ".txt"

#Write public keys to a text file
publicfile = open( public_keyname, "w")
publicfile.write( str(N) + '\n' + str(e) )
publicfile.close()

#Write private keys to a text file
privatefile = open( private_keyname, "w")
privatefile.write( str(p) + '\n' + str(q) + '\n' + str(e) + '\n' + str(d) )
privatefile.close()

#Wait for user input to close terminal window
print( '\n RSA public and private key generation complete. ')
input("\n Press return to close ")

#End of code
